#include <iostream>
#include <stdlib.h>
#include <time.h>
#include <cmath>
#include <fstream>

#ifdef __APPLE__
	#include <GLUT/glut.h>
#else
	#include <GL/glut.h>
#endif

using namespace std;

void renderPlanets(void);
void renderScene(void);
void iterEuler();
void iterVerlet();

typedef struct
{
	float x;
	float y;
	float z;
}
particle;

typedef struct
{
		double r[3];
		double rOld[3];
		double v[3];
		double m;
		double rot;
		long lastcross;
		bool lastsign;
		
}
body;

////////////////////////////////////
//   Physik                       //
////////////////////////////////////

int const N = 2000; 
double	timestep=0.01;
int		interval = 5;
int		intervalRotation = 2000;

int const MAX_PARTICLES = 10;
int const BIG_PLANETS=2;
particle particles[N][MAX_PARTICLES];
body bodys[N];
long particles_curr=0;

void initParticles(){
	bodys[0].r[0]=0.;
	bodys[0].r[1]=0.;
	bodys[0].r[2]=0.;
	bodys[0].v[0]=0.;
	bodys[0].v[1]=0.;
	bodys[0].v[2]=0.;
	bodys[0].m=1.;
	bodys[1].r[0]=0.4;
	bodys[1].r[1]=0.;
	bodys[1].r[2]=0.;
	bodys[1].v[0]=0.;
	bodys[1].v[1]=1.57;
	bodys[1].v[2]=0.;
	bodys[1].m=0.005;
/*	r[2][0]=0.8;
	r[2][1]=0.;
	r[2][2]=0.;
	v[2][0]=0.;
	v[2][1]=1.124;
	v[2][2]=0.;
	m[2]=.02;*/
	srand(time(NULL));
	for(int i=BIG_PLANETS;i<N;i++){
		double radius=0.2+(double)rand()/(double)RAND_MAX*1.;
		double phi = (double)rand()/(double)RAND_MAX*2.*M_PI;
		double vabs= sqrt(1./radius) *((double)rand()/(double)RAND_MAX*0.01+1.);;
		bodys[i].m=0.00001;
		bodys[i].r[0]=radius*cos(phi);
		bodys[i].r[1]=radius*sin(phi);
		bodys[i].r[2]=0.;
		bodys[i].v[0]=vabs*cos(phi+M_PI/2.);
		bodys[i].v[1]=vabs*sin(phi+M_PI/2.);
		bodys[i].v[2]=0.;
	}
}


void initPhysics(){
	initParticles();
	// Remove center of mass motion
	double momentum[3]={0.,0.,0.};
	double mass=0;
	for(int i=0;i<BIG_PLANETS;i++){
		for(int k=0;k<3;k++){
			momentum[k]+=bodys[i].m*bodys[i].v[k];
		}
		mass+=bodys[i].m;
	}
	for(int i=0;i<N;i++){
		for(int k=0;k<3;k++){
			bodys[i].v[k] -= momentum[k]/mass;
		}
	}
	for(int i=0;i<N;i++){
		for(int k=0;k<3;k++){
			bodys[i].rOld[k] = bodys[i].r[k];
		}
	}
	
	iterEuler();
	
	
}

void iterParticles(){
	for(int i=0;i<N;i++){
		particles[i][particles_curr].x=bodys[i].r[0];
		particles[i][particles_curr].y=bodys[i].r[1];
		particles[i][particles_curr].z=bodys[i].r[2];
	}
	particles_curr++;
	particles_curr%=MAX_PARTICLES;
}



void iterVerlet(){ 
	double rOldt[N][3];
	for(int i=0;i<N;i++){
		for(int k=0;k<3;k++){
			rOldt[i][k]=bodys[i].r[k];
		}
	}
	for(int i=0;i<N;i++){
		double a_temp[3]={0.,0.,0.};
		//Alles bist auf die ersten BIG_PLANETS Teilchen sollen Testteilchen sein:
		for(int j=0;j<BIG_PLANETS;j++){
			if(i!=j){
				double rijtemp = 0;
				for (int k=0;k<3;k++){
					rijtemp+=pow(bodys[i].r[k]-bodys[j].r[k],2);
				}
				rijtemp=pow(rijtemp,3./2.);
				for(int k=0;k<3;k++){
					a_temp[k]+=bodys[j].m/rijtemp*(rOldt[j][k]-rOldt[i][k]);
				}
				
			}
		}
		for(int k=0;k<3;k++){
			bodys[i].r[k]=2.*bodys[i].r[k]-bodys[i].rOld[k]+timestep*timestep*a_temp[k];
		}
	}
	for(int i=0;i<N;i++){
		for(int k=0;k<3;k++){
			bodys[i].rOld[k]=rOldt[i][k];
		}
	}
}



void iterEuler(){ 
	double rOldt[N][3];
	for(int i=0;i<N;i++){
		for(int k=0;k<3;k++){
			rOldt[i][k]=bodys[i].r[k];
		}
	}
	for(int i=0;i<N;i++){
		double a_temp[3]={0.,0.,0.};
		for(int j=0;j<N;j++){
			if(i!=j){
				double rijtemp = 0;
				for (int k=0;k<3;k++){
					rijtemp+=pow(bodys[i].r[k]-bodys[j].r[k],2);
				}
				rijtemp=pow(rijtemp,3./2.);
				for(int k=0;k<3;k++){
					a_temp[k]+=bodys[j].m/rijtemp*(rOldt[j][k]-rOldt[i][k]);
				}
				
			}
		}
		for(int k=0;k<3;k++){
			bodys[i].v[k]+=timestep*a_temp[k];
			bodys[i].r[k]+=timestep*bodys[i].v[k];
		}
	}
}

void iterRotation(){
	for(int i=0;i<N;i++){
		if(bodys[i].lastsign&&(bodys[i].r[1]>0)){
			bodys[i].rot=bodys[i].lastcross*timestep;
			bodys[i].lastcross=0;
			bodys[i].lastsign=false;
		}else{
			bodys[i].lastcross++;
			if(bodys[i].r[1]<0){
				bodys[i].lastsign=true;
			}
		}
	}
}

void timerStep(int value){
	
	//r[0][1]-=0.01;
	//iterEuler();
	iterVerlet();
	iterParticles();
	iterRotation();
	
	renderScene();	
	glutTimerFunc(interval,timerStep,0);
}


void timerRotation(int value){
	ofstream outRotation("rotation.dat");
	double normalRotation= bodys[1].rot;
	long normalCount = (long)( 30.*bodys[1].rot/timestep);
	for(int i=0;i<N;i++){
		if (bodys[i].rot>0&&bodys[i].lastcross<normalCount){
			outRotation << bodys[i].rot/normalRotation << endl;
		}
	}
	glutTimerFunc(intervalRotation,timerRotation,0);
}

/////////////////////////////////////
//   Rendering                     //
/////////////////////////////////////


void renderParticles(){
	glLoadIdentity();
	glColor3f(.1,.1,.2);
	glBegin(GL_POINTS);
	for(int i=0;i<N;i++){
		for(int j=0;j<MAX_PARTICLES;j++){
			glVertex3f(particles[i][j].x,particles[i][j].y,particles[i][j].z);	
		}
	}
	
	glEnd();
	glColor3f(1.,1.,1.);
}

void renderPlanets(void){
	for(int i=2;i<N;i++){
	glLoadIdentity();
	glTranslatef(bodys[i].r[0],bodys[i].r[1],bodys[i].r[2]);
	glutWireSphere(0.005,3,3);
	}
	glColor3f(1.,0.,0.);
	for(int i=0;i<BIG_PLANETS;i++){
		glLoadIdentity();
		glTranslatef(bodys[i].r[0],bodys[i].r[1],bodys[i].r[2]);
		glutWireSphere(0.01,10,10);
	}
	glColor3f(1.,1.,1.);
}




void renderScene(void){
	glClear(GL_COLOR_BUFFER_BIT);
	glLoadIdentity();
	renderPlanets();
	renderParticles();
	glFlush();
	glutSwapBuffers();
}

void renderInit(){
	glutInitDisplayMode(GLUT_DEPTH|GLUT_DOUBLE|GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(600,600);
	glutCreateWindow("nbody chaos");
	glutDisplayFunc(renderScene);

	
}


int main (int argc, char * argv[]) {
    // insert code here...
    std::cout << "Starting engine..,\n";
	initPhysics();
	glutTimerFunc(interval,timerStep,0);
	glutTimerFunc(intervalRotation,timerRotation,0);
	
	
	glutInit(&argc, argv);
	renderInit();
	
	
	
	
	glutMainLoop();
	
    return 0;
}
